/* ========================================================================= */
/* VARIABLES =============================================================== */
/* ========================================================================= */

var stackColors = [],
	checkedBoxes = 0,
	round = [],
	results = [],
	tour;

_.times(9, function (index) { stackColors.push('stack-color-' + index); });

/* ========================================================================= */
/* HELPERS ================================================================= */
/* ========================================================================= */

Handlebars.registerHelper('getCategory', function (id) {
	return _.findWhere(categories, {_id: id});
});

Handlebars.registerHelper('getItem', function (id) {
	return _.findWhere(items, {_id: id});
});

Handlebars.registerHelper('equals', function(a,b){
	return a == b;
});

Handlebars.registerHelper('notEmpty', function (a, b, c) {
	return a.length || b.length || c.length;
});

Handlebars.registerHelper('size', function (element) {
	return element.length;
});

Handlebars.registerHelper('getRows', function (items) {
	return Math.ceil(items.length / 3);
});

var trimInput = function(value) {
	return value.replace(/^\s*|\s*$/g, '');
};

var hasClass = function(element, cls) {
    return (' ' + element.className + ' ').indexOf(' ' + cls + ' ') > -1;
};

var randomIntFromInterval = function(min, max) {
	return Math.floor( Math.random() * (max - min +1) + min);
};

/* ========================================================================= */
/* FUNCTIONS =============================================================== */
/* ========================================================================= */

var siteHandler = {
	onDragMove: function(draggie) { // dragMove generico
		$(draggie.element).css({'z-index':'1051'});
	},
	onDragEnd: function(draggie) { // dragEnd generico
		$(draggie.element).css({'z-index':'1'});
	},
	/**
	 * Permite que se puedan arrastrar los items y las categorías
	 */
	initDraggabilly: function() {
		var categories = document.querySelectorAll('.category');
		for (var i=0, len = categories.length; i < len; i++) {
			var item = categories[i];
			var draggie = new Draggabilly(item, {
				containment: 'body'
			});
			draggie.on('dragMove', this.onDragMove); // changes z from 1 to 1051
			draggie.on('dragEnd', this.checkBoxs);   // check if we are in a box
			draggie.on('dragEnd', this.onDragEnd);   // changes z from 1051 to 1
		}

		var items = document.querySelectorAll('.item');
		for (var i=0, len = items.length; i < len; i++) {
			var item = items[i];
			var draggie = new Draggabilly(item);
			draggie.on('dragMove', this.onDragMove); // changes z from 1 to 1051
			draggie.on('dragEnd', this.checkBoxs);   // check if we are in a box
			draggie.on('dragEnd', this.onDragEnd);   // changes z from 1051 to 1
		}
	},
	/**
	 * Muestra los items con una animación aleatoria de uno en uno cada ciertos ms.
	 * Una vez cargados activa la función de arrastrar
	 */
	loadItems: function(callback) {
		var items = document.querySelectorAll('.item'),
			effects = ['fadeInUpBig', 'fadeInRightBig', 'fadeInDownBig', 'fadeInLeftBig' ];

		_.each(items, function (item, index) {
			var random = randomIntFromInterval(0, effects.length -1);

			setTimeout(function() {
				$(item).addClass('animated ' + effects[random]).removeClass('hidden');
			}, 200 * index);

			// Al terminar elimina las clases de la animación (con un retardo)
			if (index == items.length -1) {
				setTimeout(function() {
					$('.item').removeClass('animated ' + effects.join(' '));
					siteHandler.initDraggabilly();
					if (callback)
						callback();
				}, 200 * index + 1000);
			}
		});
	},
	checkBoxs: function(draggie, event, pointer) {
		var leftBorder1 = box0.offsetLeft;
		var rightBorder1 = leftBorder1 + box0.clientWidth +30;

		var leftBorder2 = box1.offsetLeft;
		var rightBorder2 = leftBorder2 + box1.clientWidth +30;

		var leftBorder3 = box2.offsetLeft;
		var rightBorder3 = leftBorder3 + box2.clientWidth +30;

		var leftBorder4 = box3.offsetLeft;
		var rightBorder4 = leftBorder4 + box3.clientWidth +30;

		var boxTop = boxesWrapper.offsetTop +50;
		var boxBottom = boxTop + $('#box0').height() +30;
		var boxId;
		var draggieElement = draggie.element;

		var isClosed = false;

		// Check if target
		if(pointer.pageY > boxTop  && pointer.pageY < boxBottom){ //if the pointer is in the same height than boxes...
			if (pointer.pageX > leftBorder1 && pointer.pageX < rightBorder1){
				//code for box1
				if ($('#box0').hasClass('closed'))
					isClosed = true;

				boxId = $('#box0').attr('data-id');
			} else if (pointer.pageX > leftBorder2 && pointer.pageX < rightBorder2){
				//code for box2
				if ($('#box1').hasClass('closed'))
					isClosed = true;

				boxId = $('#box1').attr('data-id');
			} else if (pointer.pageX > leftBorder3 && pointer.pageX < rightBorder3){
				//code for box3
				if ($('#box2').hasClass('closed'))
					isClosed = true;

				boxId = $('#box2').attr('data-id');
			} else if (pointer.pageX > leftBorder4 && pointer.pageX < rightBorder4){
				//code for box4
				if ($('#box3').hasClass('closed'))
					isClosed = true;

				boxId = $('#box3').attr('data-id');
			}
			else {
				$(draggieElement).animate({top: '0', left: '0', zIndex: '0'}, 400);
			}
		}
		else { // if not target
			$(draggieElement).animate({top: '0', left: '0', zIndex: '0'}, 400);
		}

		// Si la caja está cerrada no acepta más items porque ha sido comprobada
		if (isClosed){
			boxId = null;
			$(draggieElement).animate({top: '0', left: '0', zIndex: '0'}, 400);
		}

		// Si el elemento ha sido arrastrado a una caja y no está cerrada
		if (boxId !== undefined && boxId !== null) {
			var elementAccepted = true,
				currentCategory = (_.findWhere(boxes, {_id: boxId})).categoryId;
				
			// Si el elemento es una categoría, actualizamos la categoría de la caja.
			if ( $(draggieElement).hasClass('category') ) {
				var category = (_.findWhere(categories, {_id: draggieElement.id}));
				// time to check if we had a previous categoryId
				if (currentCategory !== '') {
					$('#' + currentCategory).show();
				}

				if(currentCategory != category._id)
				{	
					// Similar a "update $set" de mongo...
					_.each(boxes, function (box, index) {
						if (box._id == boxId) {
							box.categoryId = draggieElement.id;

							// Si la categoría es special folder creamos las carpetas básicas con colores aleatorios
							if (category.special && category.special.folder) {
								var randomColor;

								_.each(category.special.folder, function (folder) {
									do {
										randomColor = stackColors[_.random(0, stackColors.length-1)];
									}
									while ( _.findWhere(boxes[index].folders, {stackColor: randomColor}) !== undefined );

									box.folders.push({_id: _.uniqueId('folder_'), name: folder.name, content: [], stackColor: randomColor});
								});
							}
							else {
								box.folders = [];
							}
						}
					});
					siteHandler.areBoxesAssigned();
				}

			}
			// Si el elemento es un item
			else if ( $(draggieElement).hasClass('item') ) {
				// Si la caja tiene asignada una categoría lo insertamos en el array de items de la caja.
				if (currentCategory !== '') {
					// Simirar a "update $addToSet" de mongo...
					_.each(boxes, function (box) {
						// Si es la caja y el item no existe en la caja (fix cuando se hacia click en la caja al soltar el item)
						if ( box._id == boxId && !(_.findWhere(box.items, {id: draggieElement.id})) )
							box.items.push({id: draggieElement.id});
					});
				}
				else {
					$(draggieElement).animate({top: '0', left: '0', zIndex: '0'}, 400);
					elementAccepted = false;
				}
			}

			// Si el elemento arrastrado es una categoría ó un item en caso de que la caja tenga categoría
			if (elementAccepted) {
				// Volvemos a recargar el template boxes (falsa reactividad...) y establecemos los eventos
				siteHandler.loadTemplate('boxes');
				siteHandler.setEvents.clickBox();
				// Ocultamos el elemento
				siteHandler.hideAndReturn(draggieElement);
			}
		}
	},
	/**
	 * Devuelve el index de la caja dado su _id
	 *
	 * @param  {String} boxId 	- Id de la caja
	 * @return {Int}       		- Index o undefined en caso de no existir
	 */
	getBoxIndex: function(boxId) {
		return (_.findWhere(boxes, {_id: boxId})) ? (_.findWhere(boxes, {_id: boxId})).index : undefined;
	},
	/**
	 * Comprueba si una carta colisiona con otra o con un grupo (solo se permite que colisone con un elemento).
	 * Si colisiona con una carta crea un nuevo grupo con los ids de las cartas y lo inserta,
	 * si colisiona con un grupo y no está completo inserta el id la carta en el grupo.
	 *
	 * @param  {Object} draggie 	- Elemento que ha sido arrastrado
	 * @param  {Event} event   		- Los eventos originales mousedown o touchstart
	 */
	checkGroup: function(draggie, event) {
		var elementCoords = [],
			currentElementId = $(draggie.element).attr('data-box-item-id'),
			currentBoxId = $(draggie.element).parents('.modal-content').find('#checkItems').attr('data-box-id'),
			box = _.findWhere(boxes, {_id: currentBoxId}),
			currentBoxIndex = siteHandler.getBoxIndex(currentBoxId),
			elements = document.querySelectorAll('.element-group'),
			isInFolder = $(draggie.element).parents('.modal-body').hasClass('folder');
			maxSize = parseInt($(draggie.element).parents('.modal-body').attr('data-max-size'));

			// Almacenamos las coordenadas de cada elemento (item o stack)
			_.each(elements, function (element) {
				var coords = {
					left: element.offsetLeft,
					right: element.offsetLeft + element.clientWidth,
					top: element.offsetTop,
					bottom: element.offsetTop + element.clientHeight
				};

				if ( hasClass(element, 'folder') )
					_.extend(coords, {folderId: element.getAttribute('data-folder-id')});
				else if ( hasClass(element, 'stack') )
					_.extend(coords, {stackId: element.getAttribute('data-stack-id')});
				else
					_.extend(coords, {itemId: element.getAttribute('data-box-item-id')});

				elementCoords.push(coords);
			});

			// Si el item arrastrado colisiona con otros elementos guardamos sus ids
			var currentElement = _.findWhere(elementCoords, {itemId: currentElementId}),
				collisions = [];
			_.each(elementCoords, function (element) {
				if ( currentElementId != element.itemId && siteHandler.collisionExists(currentElement, element) ) {
					if (element.folderId)
						collisions.push({ type: 'folder', id: element.folderId });
					else if (element.itemId)
						collisions.push({ type: 'item', id: element.itemId });
					else
						collisions.push({ type: 'stack', id: element.stackId });
				}
			});

			// Si la agrupación es con un único elemento (item, stack o folder)
			if (collisions.length == 1) {

				var toRemove = [];

				// Si es con un item creamos un nuevo grupo con los dos items
				if (collisions[0].type == 'item') {
					// Si no es el tipo folder podemos unir items
					if (!isInFolder) {
						var randomColor,
							newGroup = {
							_id: _.uniqueId('group_'),
							content: [collisions[0].id, currentElementId]
						};

						// Generamos un color aleatorio (sin repetir) para el grupo
						do {
							randomColor = stackColors[_.random(0, stackColors.length-1)];
						}
						while ( _.findWhere(boxes[currentBoxIndex].groups, {stackColor: randomColor}) !== undefined );

						newGroup.stackColor = randomColor;

						boxes[currentBoxIndex].groups.push(newGroup);
						toRemove = newGroup.content;
					}
					else {
						$(draggie.element).animate({top: '0', left: '0', zIndex: '0'}, 400);
					}
				}
				// Si es con una carpeta comprobamos si tiene min/max/equals para ver si podemos insertar
				else if (collisions[0].type == 'folder') {
					var folderIndex;

					// Buscamos la carpeta
					_.each(boxes[currentBoxIndex].folders, function (folder, index) {
						// Si la carpeta tiene límite máximo y si no está completa
						if (folder._id == collisions[0].id && maxSize) {
							if (folder.content.length < maxSize)
								folderIndex = index;
						}
						// Si la carpeta no tiene límite
						else if (folder._id == collisions[0].id) {
							folderIndex = index;
						}
					});

					// Si la encontramos copiamos el item en la carpeta y prepraramos su id para eliminarlo
					if (folderIndex !== undefined) {
						boxes[currentBoxIndex].folders[folderIndex].content.push(currentElementId);
						toRemove.push(currentElementId);
					}
					else {
						$(draggie.element).animate({top: '0', left: '0', zIndex: '0'}, 400);
					}
				}
				// Si es con un grupo y no está completo lo insertamos en el grupo
				else {
					var groupIndex;

					// Buscamos el grupo
					_.each(boxes[currentBoxIndex].groups, function (group, index) {
						if (group._id == collisions[0].id && group.content.length < maxSize) {
							groupIndex = index;
						}
					});

					// Si lo encontramos copiamos el item en el grupo y preparamos su id para eliminarlo
					if (groupIndex !== undefined) {
						boxes[currentBoxIndex].groups[groupIndex].content.push(currentElementId);
						toRemove.push(currentElementId);
					}
					else {
						$(draggie.element).animate({top: '0', left: '0', zIndex: '0'}, 400);
					}
				}

				// Eliminamos sus ids del array items de la caja
				_.each(toRemove, function (itemId) {
					boxes[currentBoxIndex].items = _.reject(boxes[currentBoxIndex].items, function (item) { return item.id == itemId; });
				});

				// Renderizamos el contenido del modal para mostrar los grupos (si se han creado)
				if (toRemove.length !== 0) {
					siteHandler.loadTemplate('box-content-body', '.modal-body', boxes[currentBoxIndex]);
					siteHandler.setEvents.dragItemsToGroup();
					siteHandler.setEvents.removeItemFromBox(boxes[currentBoxIndex]);
				}
			}
			else {
				// Devolver a la posición
				$(draggie.element).animate({top: '0', left: '0', zIndex: '0'}, 400);

			}
	},
	/**
	 * Detecta las colisiones entre dos items.
	 *
	 * @param  {Object} coords1 	- ID del item y su posición (left, right, top, bottom)
	 * @param  {Object} coords2		- ID del item y su posición (left, right, top, bottom)
	 * @return {Boolean}        	- Devuelve true si hay colisión y false en caso contrario
	 */
	collisionExists: function(coords1, coords2) {
		return (( (coords1.left >= coords2.left && coords1.left <= coords2.right) || (coords1.right >= coords2.left && coords1.right <= coords2.right) ) &&
			 	( (coords1.bottom <= coords2.bottom && coords1.bottom >= coords2.top) || (coords1.top <= coords2.bottom && coords1.top >= coords2.top) ));
	},
	hideAndReturn: function(draggieElement) {
		// divide functions if we want the effect unanimed (instant)
		$(draggieElement).fadeOut('fast', function(){
			$(draggieElement).css({'top':'0', 'left': '0'});
		});
	},
	/**
	 * Comprueba que todas las cajas tengan asignada una categoría y activa los botones para cerrar el modal de categorías
	 */
	areBoxesAssigned: function() {
		if ( ! _.findWhere(boxes, {categoryId: ''}) )
			$('.categories-icon').addClass('animated tada');
		else
			$('.categories-icon').removeClass('animated tada');
	},
	/**
	 * Comprueba si se ha terminado una ronda y el juego.
	 * Prepara los datos de la colección Results.
	 *
	 * @param  {String} categoryId		- ID de la categoría de la caja comprobada
	 * @param  {int}    score        	- Puntuación total obtenida en la caja
	 * @param  {Object} checkedElements	- Items y Grupos con el id de los items y su estado (correcto/incorrecto).
	 *                                    Ej.:
	 *                                    {
	 *                                        items: [{_id: 'itemId0', right: true}, ...],
	 *                                        groups: [{content: ['itemId1', 'itemId2', 'itemId3'], right: false}, ...]
	 *                                    }
	 */
	roundController: function(categoryId, score, checkedElements, finish) {
		// Guardamos los datos de la comprobación de la caja en el array round
		
		if(!finish){
			round.push({
				categoryId: categoryId,
				result: {
					boxScore: score,
					elements: checkedElements
				}
			});
		}
		// Comprobamos si se ha terminado una ronda, no quedan mas items o han enviado finish
		if ( (checkedBoxes % 4 === 0) || ($('#items .item').length === 0) || finish) {
			var currentRound = Math.ceil( checkedBoxes / 4 ),
			//	roundNumber = (currentRound === 0) ? 1 : currentRound;
			roundNumber = currentRound;

			if (finish && (checkedBoxes % 4 === 0))
				roundNumber++;

			// Insertamos la ronda en el array results
			results.push({boxes: round, roundNumber: roundNumber});
			// Vaciamos el array de rondas, los datos de las cajas
			round = [];
			// Vacía las cajas y las marca como abiertas. Similar a "update $set" de mongo...
			_.each(boxes, function (box) {
				_.extend(box, {categoryId: '', items: [], groups: [], folders: [], closed: false});
			});

			// Comprobamos si se ha terminado el juego o han enviado finish
			if ( checkedBoxes == categories.length || ($('#items .item').length === 0) || finish ) {

				// Si nos sobran items obtenemos sus ids y los almacenamos en resultados
				if ($('#items .item').length !== 0) {
					var unusedItems = $('#items .item'),
						unusedIds = [];
					_.each(unusedItems, function (item) {
						unusedIds.push(item.id);
					});
					results.push({unusedItems: unusedIds});
				}

				// Mostramos la pantalla de resultados. Guardando la puntuación para mostrarla cuando se renderice el template
				var currentScore = $('#currentScore').text();
				siteHandler.loadTemplate('results', '#mainWrapper', {results: results});
				$('.item-badge').css({"display":"block"});
				$('.item-score').css({"display":"block"});
				$('.total-score').text('Total: '+currentScore+' ptos');
				//
				siteHandler.setEvents.newGame();
				$('.categories-icon').bind('click',function(e){e.preventDefault();});
				$('#finishGame').bind('click',function(e){e.preventDefault();});
				siteHandler.setEvents.gameInfo();
			}
			// Si no ha terminado mostramos las cajas abiertas y el selector de categorías
			else {
				siteHandler.loadTemplate('boxes');
				siteHandler.areBoxesAssigned();
			}

			// Actualizamos las rondas en el marcador
			$('#round').text(roundNumber+1);
		}

	},
	/**
	 * Carga un template con los datos que recibe como parámetro.
	 *
	 * @param  {String}  name    	- Nombre del fichero .hbs donde se encuentra el template
	 * @param  {String}  where   	- Selector CSS donde vamos a insertar el template
	 * @param  {Object}	 context	- Objeto con los datos que va a utilizar el template
	 * @param  {Boolean} append  	- Si es true inserta el template al final de un contenedor para no borrar su contenido
	 */
	loadTemplate: function(name, where, context, append) {
		var location = where || '#' + name,
			data = context || {boxes: boxes, categories: categories, items: items},
			template = JST['app/templates/' + name + '.hbs'],
			html = template(data);

		if (append)
			$(location).append(html);
		else
			document.querySelector(location).innerHTML = html;

		// Guarda las cajas para poder obtener sus medidas y offsetLeft posteriormente
		if (name == 'boxes') {
			box0 = $('#box0')[0];
			box1 = $('#box1')[0];
			box2 = $('#box2')[0];
			box3 = $('#box3')[0];
		}
	},
	/**
	 * Establece los eventos de la app
	 */
	setEvents: {
		/**
		 * Al hacer click sobre una caja se muestra un modal con su contenido (items).
		 * Si la caja ha sido comprobada se ocultará al cerrar el modal.
		 */
		clickBox: function() {

			$('.box').on('click', function(e) {
				// If the box has the class closed, we shouldn't be able to open it
				if (!$(e.currentTarget).hasClass('closed')){

					// Obtenemos los datos de la caja
					var boxId = $(e.currentTarget).attr('data-id'),
						box = _.findWhere(boxes, {_id: boxId});
				
					// Renderizamos el modal con los datos
					siteHandler.loadTemplate('box-content', '#containerWrapper', box, true);
						var boxItems = document.querySelectorAll('.box-item');

					// Asignamos el evento para ocultar la descripción de la categoria
					siteHandler.setEvents.hideDescriptionCategory();
					


				$('.edit-remove-item').on('click', function(){
					$('.item-close').toggle();
					$('.edit-remove-item i').toggleClass('fa-eraser fa-check');
					 // Buscamos los items que pertenezcan a un group (dagables de group o folder) y los deshabilitamos.
					var boxItemsDragables = document.querySelectorAll('.item.group');
					_.each(boxItemsDragables, function (item) {
						var elem = $(item).attr('data-box-item-id');
						$('[data-box-item-id="' + elem + '"]').toggleClass('drag-disabled');
					});
				});


					siteHandler.setEvents.removeItemFromBox(box);

					// Mostramos el modal desde abajo hacía arriba
					$('#boxContent .modal').removeClass('in');
					$('#boxContent').modal();

					// Eliminamos el modal cuando se oculta
					$('#boxContent').on('hidden.bs.modal', function (event) {
						// Si la caja ha sido comprobada la ocultamos
						if ( $(event.currentTarget).hasClass('checked') ) {
							var boxId = $('#checkItems').attr('data-box-id'),
								box = _.findWhere(boxes, {_id: boxId});

							$(e.currentTarget).addClass('closed');
							( $('html').hasClass('ie9') ) ? $(e.currentTarget).find('.category').remove() : $(e.currentTarget).find('.category').addClass('animated hinge');
							$(e.currentTarget).find('.box-score').text('');

							// Marca la caja como cerrada. Similar a "update $set" de mongo...
							_.each(boxes, function (box) {
								if (box._id == boxId)
									_.extend(box, {closed: true});
							});

							// Comprueba si se ha terminado la ronda o el juego
							siteHandler.roundController(box.categoryId, score, checkedElements, false);
						}
						$(event.currentTarget).remove();
						$(this).off();
					});

					// Al abrir el modal en IE9
					if ( $('html').hasClass('ie9') ) {

						// Fallback para el evento de Bootstrap "shown.bs.modal"...
						if ( $('#boxContent').hasClass('in') ) {

							// Si el item está en una categoría edit
							if ( $('.modal-body').hasClass('edit') ) {
								// Ponemos la carta delante
								$('.innerWrapper').addClass('front-ie9');
								// Al pulsarlo mostramos/ocultamos el text area
								$('.item-container').on('click', function() {
									$innerWrapper = $(this).find('.innerWrapper');
									$innerWrapper.toggleClass('front-ie9');

									if ( !$innerWrapper.hasClass('front-ie9') )
										$innerWrapper.parent().find('.edit-text').focus();
									else
										$innerWrapper.parent().find('.edit-text').blur();
								});

								// Al tomar/perder el foco giramos la carta (en caso de tabular se girará...)
								$('.edit-text').on('focus', function(event) {
									$(this).parents('.item').find('.innerWrapper').removeClass('front-ie9');
								});
								$('.edit-text').on('blur', function(event) {
									$(this).parents('.item').find('.innerWrapper').addClass('front-ie9');
								});

								// Al cambiar el texto del item
								$('.edit-text').on('change', function(event) {
									var boxIndex = siteHandler.getBoxIndex( $(this).parents('.modal-content').find('#checkItems').attr('data-box-id') ),
										itemId = $(this).parents('.item').attr('data-box-item-id'),
										text = $(this).val();

									// Guardamos el texto en el item editado
									_.each(boxes[boxIndex].items, function (item) {
										if (item.id == itemId)
											item.edit = text;
									});
								});
							}
							// Si el item está en una categoría group o folder
							else if ( $('.modal-body').hasClass('group') || $('.modal-body').hasClass('folder') ) {
								siteHandler.setEvents.dragItemsToGroup();
							}
						}
					}
					// Para el resto de navegadores
					else {
						$('#boxContent').on('shown.bs.modal', function (event) {
							// Si el item está en una categoría edit
							if ( $('.modal-body').hasClass('edit') ) {
								// Al pulsarlo lo giramos y le ponemos/quitamos el foco
								$('.item-container').on('click', function() {
									$(this).toggleClass('flipped');

									if ( $(this).hasClass('flipped') ) {
										$(this).find('.edit-text').focus();
									}
									else {
										$(this).find('.edit-text').blur();
									}
								});

								// Al tomar/perder el foco giramos la carta (en caso de tabular se girará...)
								$('.edit-text').on('focus', function(event) {
									$(this).parents('.item-container').addClass('flipped');
								});
								$('.edit-text').on('blur', function(event) {
									$(this).parents('.item-container').removeClass('flipped');
								});

								// Al cambiar el texto del item
								$('.edit-text').on('change', function(event) {
									var boxIndex = siteHandler.getBoxIndex( $(this).parents('.modal-content').find('#checkItems').attr('data-box-id') ),
										itemId = $(this).parents('.item').attr('data-box-item-id'),
										text = $(this).val();

									// Guardamos el texto en el item editado
									_.each(boxes[boxIndex].items, function (item) {
										if (item.id == itemId)
											item.edit = text;
									});
								});
							}
							// Si el item está en una categoría group o folder
							else if ( $('.modal-body').hasClass('group') || $('.modal-body').hasClass('folder') ) {
								siteHandler.setEvents.dragItemsToGroup();
							}
						});
					}
				}

				// Establecemos los eventos de nuevo para que funcione el botón de comprobar del modal
				siteHandler.setEvents.clickCheckItems();
			});
		},
		/**
		 * Al entregar una caja se comprueban los items correctos e incorrectos (verde/rojo).
		 * Se vuelven a mostrar los items incorrectos en el tablero y se eliminan los correctos.
		 */
		clickCheckItems: function() {

			$('#checkItems').on('click', function(e) {
				$('.edit-remove-item').off('click');
				$('.item-close').hide();
				$('.item-badge').css({"display":"block"});
				$('.item-score').css({"display":"block"});
				// Obtenemos los datos de la caja
				var boxId = $(e.currentTarget).attr('data-box-id'),
					box = _.findWhere(boxes, {_id: boxId}),
					currentCategory = (_.findWhere(categories, {_id: box.categoryId})),
					categoryScore = currentCategory.score,
					currentScore = parseInt($('#currentScore').text());

				$('.item-container, .stack-indicator').unbind('click');
				$(e.currentTarget).parents('.modal-content').find('.modal-body').addClass('feedback');

				// Variables globales para poder usarlas en clickBox()
				score = 0;
				checkedElements = {
					items: [],
					groups: [],
					folders: []
				};
				// Bloqueamos el botón
				$(e.currentTarget).attr('disabled', 'disabled');

				// Recorremos los items de la caja... para categoría simple, edit y group (elementos no agrupados)
				_.each(box.items, function (item) {
					var right = false; // Suponemos que el item es incorrecto
					
					// Obtenemos las categorías a las que pertenece el item
					var itemCategoriesId = (_.findWhere(items, {_id: item.id})).categoriesId;

					// Obtenemos si la categoría es especial
					var special = currentCategory.special;

					// Si el item está en una categoría especial
					if (special) {
						var currentItem = (_.findWhere(items, {_id: item.id}));
						
						// Si la clase especial es edit
						if (special.edit) {
							// si el item pertenece a la categoría
							if(itemCategoriesId.indexOf(box.categoryId) != -1){
								var	itemSpecialEdit = ( _.has(currentItem, 'special') ) ? currentItem.special.edit : null;
									editValue = $('[data-box-item-id="' + item.id + '"]').find('.edit-text').val();

								// Gira la carta para ver el resultado
								$('[data-box-item-id="' + item.id + '"]').parents('.item-container').removeClass('flipped');

								// Si el valor escrito coincide
								if ( itemSpecialEdit && itemSpecialEdit.indexOf( trimInput(editValue.toLowerCase()) ) != -1 )
									right = true;
							}
						}
					}
					// Si el item pertenece a la categoría
					else if (itemCategoriesId.indexOf(box.categoryId) != -1) {
						right = true;
					}

					// Deshabilitamos el item para que no se pueda arrastrar
					$('[data-box-item-id="' + item.id + '"]').addClass('drag-disabled');

					// Si el item pertenece a la categoría y/o está correcto
					if (right) {
						// Destacamos el item como correcto
						$('[data-box-item-id="' + item.id + '"]').addClass('correct').find('.item-badge .fa').addClass('fa-check');
						$('[data-box-item-id="' + item.id + '"]').find('.item-badge').addClass('animated bounceIn');
						$('[data-box-item-id="' + item.id + '"]').find('.item-score').text('+'+categoryScore);
						score += categoryScore;

						// Eliminamos el item (oculto) del tablero
						$('#' + item.id).remove();
					}
					else {
						// Destacamos el item como erroneo
						$('[data-box-item-id="' + item.id + '"]').addClass('incorrect').find('.item-badge .fa').addClass('fa-times');
						$('[data-box-item-id="' + item.id + '"]').find('.item-badge').addClass('animated bounceIn');
						$('[data-box-item-id="' + item.id + '"]').find('.item-score').text('-1');
						score--;

						// Volvemos a mostrarlo en el tablero
						$('#' + item.id).css({'top': 'initial', 'left': 'initial', 'z-index': '0'}).show();
					}

					// Insertamos el item actual con su estado (correcto/incorrecto)
					checkedElements.items.push({_id: item.id, right: right});
				});

				// Recorremos los grupos de la caja
				_.each(box.groups, function (boxGroup, index) {
					var right = false;

					// Comprobamos si los grupos son correctos...
					// Si por defecto está correcto
					if (currentCategory.special.allRight) {
						// Si no hay mínimo ó si cumple con el mínimo
						if (!currentCategory.special.minGroupSize || boxGroup.content.length >= currentCategory.special.minGroupSize)
							right = true;
					}
					// Si "valen todos menos" comprobamos que no estén en el grupo
					else if (currentCategory.special.excludeValues) {
						// _.flatten convierte un array anidado en un array simple. http://underscorejs.org/#flatten
						// En este caso también podemos usar currentCategory.special.group[0] porque solo habrá un elemento
						if ( (_.intersection(_.flatten(currentCategory.special.group), boxGroup.content)).length === 0 ) {
							// Si no hay mínimo ó si cumple con el mínimo
							if (!currentCategory.special.minGroupSize || boxGroup.content.length >= currentCategory.special.minGroupSize)
								right = true;
						}
					}
					// Si no, comparamos con los de las categorías
					else {
						_.each(currentCategory.special.group, function (group) {
							if ( (_.difference(group, boxGroup.content)).length === 0 )
								right = true;
						});
					}

					// Si el grupo actual es correcto
					if (right) {
						$('[data-stack-id="' + boxGroup._id + '"]').addClass('active correct').find('.stack-badge .fa').addClass('fa-check');
						$('[data-stack-id="' + boxGroup._id + '"]').find('.stack-score').text('+'+categoryScore);
						score += categoryScore;

						// Eliminamos los items (ocultos) del tablero
						_.each(boxGroup.content, function (itemId) {
							$('#' + itemId).remove();
						});
					}
					else {
						$('[data-stack-id="' + boxGroup._id + '"]').addClass('active incorrect').find('.stack-badge .fa').addClass('fa-times');
						$('[data-stack-id="' + boxGroup._id + '"]').find('.stack-score').text('-1');
						score--;

						// Volvemos a mostrar los items del grupo en el tablero
						_.each(boxGroup.content, function (itemId) {
							$('#' + itemId).css({'top': 'initial', 'left': 'initial', 'z-index': '0'}).show();
						});
					}
					// Ocultamos el icono de desagrupar
					$('[data-stack-id="' + boxGroup._id + '"]').find('.break-stack').addClass('hidden');

					// Preparamos resultados
					checkedElements.groups.push({content: boxGroup.content, right: right, stackColor: boxGroup.stackColor});
				});

				// Recorremos las carpetas de la caja
				_.each(box.folders, function (boxFolder, index) {
					var right = false,
						folderName = boxFolder.name,
						folderResults = [],
						correctFolder = false,
						folderScore = 0;

					// Comprobamos item por item de la carpeta para ver si son correctos
					_.each(boxFolder.content, function (item) {
						var folder = _.findWhere(currentCategory.special.folder, {name: folderName});

						// Si el item es incorrecto lo destacamos y volvermos a mostrarlo en el tablero
						if (_.indexOf(folder.content, item) == -1) {
							$('[data-folder-item="' + item + '"]').addClass('wrong');
							$('#' + item).css({'top': 'initial', 'left': 'initial', 'z-index': '0'}).show();
							folderResults.push({ itemId: item, right: false });
						}
						// Si es correcto eliminamos el item (oculto) del tablero
						else {
							$('#' + item).remove();
							folderResults.push({ itemId: item, right: true });
						}
					});

					// Guardamos el número de items correctos e incorrectos
					var itemResults = _.countBy(folderResults, function(folder) { return folder.right ? 'right': 'wrong'; });
					if (!itemResults.right) { itemResults.right = 0; }
					if (!itemResults.wrong) { itemResults.wrong = 0; }

					// Comprobamos si la caja está correcta para...
					// Igual a X items (min y max) -> Deben estar todos bien
					if (currentCategory.special.minGroupSize && currentCategory.special.maxGroupSize && itemResults.right == currentCategory.special.equalsGroupSize)
						correctFolder = true;
					// Mínimo X items -> Debe estar el mínimo bien
					else if (currentCategory.special.minGroupSize && itemResults.right >= currentCategory.special.minGroupSize)
						correctFolder = true;
					// Máximo X items -> Debe estar al menos uno bien
					else if (currentCategory.special.maxGroupSize && itemResults.right >= 1) {
						correctFolder = true;
					}

					// Si la carpeta actual es correcta
					if (correctFolder) {
						$('[data-folder-id="' + boxFolder._id + '"]').addClass('active correct').find('.stack-badge .fa').addClass('fa-check');
						$('[data-folder-id="' + boxFolder._id + '"]').find('.stack-score').text('+' + (categoryScore*itemResults.right - 1*itemResults.wrong));
						folderScore = (categoryScore*itemResults.right - 1*itemResults.wrong);
						score += folderScore;
					}
					else {
						$('[data-folder-id="' + boxFolder._id + '"]').addClass('active incorrect').find('.stack-badge .fa').addClass('fa-times');
						$('[data-folder-id="' + boxFolder._id + '"]').find('.stack-score').text('-1');
						folderScore = -1;
						if(boxFolder.content.length == 0 ){
							folderScore = 0;
							$('[data-folder-id="' + boxFolder._id + '"]').find('.stack-score').text('0');	
						}else
							score--;
					}

					// Preparamos los resultados
					checkedElements.folders.push({
						name: boxFolder.name,
						stackColor: boxFolder.stackColor,
						right: correctFolder,
						content: folderResults,
						folderScore: folderScore
					});
				});

				// Marcamos la caja como comprobada e incrementamos el número de cajas comprobadas
				$('#boxContent').addClass('checked');
				checkedBoxes++;

				// Actualizamos la puntuación en el marcador
				if (currentScore + score < 0)
					$('#currentScore').text('0');
				else
					$('#currentScore').text(currentScore + score);

				$(this).off();
			});
		},
		/**
		 * Muestra el modal de las categorías y cambia el estilo de los elementos al mostrarlo u ocultarlo
		 */
		clickCategories: function() {
			// Cuando hacemos click en el icono de categorías
			$('.categories-icon').on('click', function (e) {
				e.preventDefault();
				$('.controllers').toggleClass('hide-controler');
				$(this).find('.fa').toggleClass('fa-list').toggleClass('fa-th');
				$('.items-categories-wrapper').toggleClass('translated').animate({scrollTop: 0}, 800);

			});
		},
		/**
		 * Permite arrastrar los items que tengan la clase .group para formar grupos
		 */
		dragItemsToGroup: function() {
			var itemsGroup = document.querySelectorAll('.item.group');
			_.each(itemsGroup, function (item) {
				var draggie = new Draggabilly(item, {containment: '.modal-body'});
				draggie.on('dragMove', siteHandler.onDragMove);	// changes z from 1 to 1051
				draggie.on('dragEnd', siteHandler.onDragEnd);	// changes z from 1051 to 1
				draggie.on('dragEnd', siteHandler.checkGroup);	// check if we are in a box
			});

			siteHandler.setEvents.itemGroupEvents();
		},
		/**
		 * Establece los eventos del grupo de items
		 */
		itemGroupEvents: function() {
			// Muestra u oculta el contenido de un grupo de items
			$('.stack-indicator').on('click', function(event) {
				if ( !$(this).parents('.group').hasClass('feedback') )
					$(this).parent().toggleClass('active');
			});

			// Deshace un grupo de items en items sencillos
			$('.break-stack').on('click', function(event) {
				var stackId = $(this).parent().attr('data-stack-id'),
					boxId = $(this).parents('.modal-content').find('#checkItems').attr('data-box-id'),
					boxIndex = siteHandler.getBoxIndex(boxId),
					itemsId = (_.findWhere(boxes[boxIndex].groups, {_id: stackId})).content;

				// Insertamos los items en la caja sueltos
				_.each(itemsId, function (itemId) {
					boxes[boxIndex].items.push({id: itemId});
				});

				// Eliminamos el grupo de la caja
				boxes[boxIndex].groups = _.reject(boxes[boxIndex].groups, function (group) { return group._id == stackId; });

				// Renderizamos el contenido del template para ver los items desagrupados
				siteHandler.loadTemplate('box-content-body', '.modal-body', boxes[boxIndex]);
				siteHandler.setEvents.dragItemsToGroup();
				siteHandler.setEvents.removeItemFromBox(boxes[boxIndex]);
			});

			// Vacía una carpeta de items
			$('.break-folder').on('click', function(event) {
				var folderId = $(this).parent().attr('data-folder-id'),
					boxId = $(this).parents('.modal-content').find('#checkItems').attr('data-box-id'),
					boxIndex = siteHandler.getBoxIndex(boxId),
					itemsId = (_.findWhere(boxes[boxIndex].folders, {_id: folderId})).content;

				// Insertamos los items en la caja sueltos
				_.each(itemsId, function (itemId) {
					boxes[boxIndex].items.push({id: itemId});
				});

				// Vaciamos la carpeta
				_.each(boxes[boxIndex].folders, function (folder) {
					if (folder._id == folderId)
						folder.content = [];
				});

				// Renderizamos el contenido del template para ver los items desagrupados
				siteHandler.loadTemplate('box-content-body', '.modal-body', boxes[boxIndex]);
				siteHandler.setEvents.dragItemsToGroup();
				siteHandler.setEvents.removeItemFromBox(boxes[boxIndex]);
			});
		},
		/**
		 * Carga e inicia el tour
		 */
		selectTourOption: function() {
			$('#tour').on('click', function() {
				siteHandler.shepherd();
				tour.start();
			});
		},
		selectPlayOption: function(){
			$('#play').on('click', function() {
				$('#selectTour').modal('hide');
				siteHandler.setEvents.clickCategories();
				siteHandler.setEvents.newGame();
				siteHandler.setEvents.finishGame();
				siteHandler.setEvents.gameInfo();
			});
		},
		hideDescriptionCategory: function(){
			$('.icon-description-category').on('click', function(e){

				if($(e.currentTarget).hasClass('fa-minus')) {
					$(e.currentTarget).removeClass('fa-minus').addClass('fa-plus');
					$(e.currentTarget).parents('.modal-header').find('.span-description').fadeOut();
				}
				else {
					$(e.currentTarget).removeClass('fa-plus').addClass('fa-minus');
					$(e.currentTarget).parents('.modal-header').find('.span-description').fadeIn();
				}
			});
		},
		removeItemFromBox: function(box){

			$('.item-close').on('click', function(e){
				e.stopPropagation();
				var itemId =$(e.currentTarget).parents('.item').attr('data-box-item-id');
				var itemPos=-1;
				_.each(box.items, function (item, index) {
					if(item.id == itemId)
						itemPos = index;
				});
				box.items.splice(itemPos, 1);
				$('[data-box-item-id="' + itemId + '"]').remove();
				$('#' + itemId).css({'top': 'initial', 'left': 'initial', 'z-index': '0'}).show();
			});
		},
		finishGame: function(){
			$('#finishGame').off('click');
			$('#finishGame').on('click', function(e){
				e.preventDefault();
				siteHandler.roundController(null, 0, null, true);
			});
		},
		newGame: function(){

			$('#newGame').off('click');
			$('#newGame').on('click', function(e){
				e.preventDefault();
				checkedBoxes = 0;
				round = [];
				results = [];
				boxes = [
					{ categoryId: '', items: [], groups: [], folders: [] },
					{ categoryId: '', items: [], groups: [], folders: [] },
					{ categoryId: '', items: [], groups: [], folders: [] },
					{ categoryId: '', items: [], groups: [], folders: [] }
				];

				// Asignamos un id único y un index a cada caja
				_.each(boxes, function (box, index) {
					_.extend(box, {_id: _.uniqueId('box_'), index: index});
				});
				
				siteHandler.initGame(false);

			});
		},
		gameInfo: function(){
			$('#game-info').off('click');
			$('#game-info').on('click', function(e){
				e.preventDefault();
				siteHandler.loadTemplate('information', "#info-game .modal-content");
				$('#info-game').modal();
				$('#info-game').on('hide.bs.modal', function (event) {
						$(event.currentTarget).find('.modal-content').children().remove();
						$(this).off();
					});
			});
		}
	},
	/**
	 * Crea un tour guiado estableciendo sus propiedades, eventos y pasos.
	 */
	shepherd: function() {
		tour = new Shepherd.Tour({
			defaults: {
				classes: 'shepherd-theme-arrows'
			},
		});

		// Events
		tour.on('start', function() {
			$('body').append('<div class="modal-backdrop fade in"></div>');
		});

		tour.on('cancel', function() {
			$('.modal-backdrop').remove();
			siteHandler.setEvents.clickCategories();
			siteHandler.setEvents.newGame();
			siteHandler.setEvents.finishGame();
			siteHandler.setEvents.gameInfo();
		});

		tour.on('complete', function() {
			$('.modal-backdrop').remove();
			siteHandler.setEvents.clickCategories();
			siteHandler.setEvents.newGame();
			siteHandler.setEvents.finishGame();
			siteHandler.setEvents.gameInfo();
		});

		// Steps
		tour.addStep('controls', {
			title: 'Controles',
			text: 'En la parte superior encontrarás los controles y la información del juego.',
			attachTo: '.controllers bottom',
			showCancelLink: true,
			buttons: [
				{
					text: 'Ok',
					action: function() {
						$('.modal-backdrop').hide();
						tour.next();
					}
				}
			]
		});

		tour.addStep('elements', {
			title: 'Cartas',
			text: 'El tablero del juego se compone de una serie de cartas que tendrás que ordenar en cada cajón según la etiqueta que hayas elegido.',
			showCancelLink: true,
			buttons: [
				{
					text: 'Ok',
					action: function() {
						$('.modal-backdrop').show();
						tour.next();
					}
				}
			]
		});

		tour.addStep('categories', {
			title: 'Categorías',
			text: 'Haciendo click aquí puedes ver las categorías.',
			attachTo: '.categories-icon bottom',
			classes: 'shepherd-theme-arrows shepherd-categories',
			showCancelLink: true,
			buttons: [
				{
					text: 'Ok',
					action: function() {
						siteHandler.setEvents.clickCategories();
						$('.categories-icon').click();
						$('.categories-icon').unbind().bind('click',function(e){e.preventDefault();});
						$('.modal-backdrop').hide();
						tour.next();
					}
				}
			]
		});

		tour.addStep('boxes-categories', {
			title: 'Cajas',
			text: 'Elige las etiquetas y arrástralas a cada caja. Recuerda que con las categorías más difíciles obtendremos más puntos.',
			attachTo: '#boxes top',
			classes: 'shepherd-theme-arrows shepherd-boxes-categories',
			showCancelLink: true,
			buttons: [
				{
					text: 'Ok',
					action: function() {
						siteHandler.setEvents.clickCategories();
						$('.categories-icon').click();
						$('.categories-icon').unbind().bind('click',function(e){e.preventDefault();});
						$('.items-categories-wrapper').removeClass('translated');
						tour.next();
					}
				}
			]
		});

		tour.addStep('boxes-items', {
			title: 'Items',
			text: 'Después clasifica cada carta en su cajón correspondiente y ¡empieza a Acomolar!',
			attachTo: '#boxes top',
			classes: 'shepherd-theme-arrows shepherd-boxes-items',
			showCancelLink: true,
			buttons: [
				{
					text: 'Ok',
					action: function() {
						tour.next();
					}
				}
			]
		});

		tour.addStep('endGame', {
			title: 'Reiniciar o finalizar juego',
			text: 'Haciendo click aquí puedes ver las opciones de reiniciar, finalizar o información sobre el juego.',
			attachTo: '.options-right bottom',
			classes: 'shepherd-theme-arrows shepherd-endGame',
			showCancelLink: true,
			buttons: [
				{
					text: 'Jugar',
					action: function() { 
						tour.complete(); 
					}
					
				}
			]
		});
	},
	/**
	 * Inicializa el juego
	 * @param  {Boolean} tour 	- Si es true pregunta si el usuario quiere hacer el tour o jugar, false en caso contrario.
	 */
	initGame: function(tour) {
		this.loadTemplate('home', '#mainWrapper');
		this.loadTemplate('boxes');
		$('.mobile').on('click', function (e) {e.preventDefault();});
		$('.categories-icon').on('click', function (e) {e.preventDefault();});
		if (tour) {
			this.loadItems(function() {
				siteHandler.loadTemplate('select-tour', 'body', null, true);
				$('#selectTour').modal();
				siteHandler.setEvents.selectTourOption();
				siteHandler.setEvents.selectPlayOption();
			});
		}
		else {
			this.loadItems();
			this.setEvents.clickCategories();
			siteHandler.setEvents.newGame();
			siteHandler.setEvents.finishGame();
			siteHandler.setEvents.gameInfo();
		}

		var boxes = document.querySelectorAll('.box');
		boxesWrapper = document.querySelector('.boxes-wrapper'); // necesario para saber la altura de las cajas
	},
	/**
	 * Carga la pantalla de introducción y simula un delay...
	 */
	init: function() {
		this.loadTemplate('intro', "#mainWrapper", game);
		setTimeout(function() { siteHandler.initGame(true); }, 1500);
	}
};

siteHandler.init();